package de.hft.stuttgart.citydoctor2.webservice.database;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import de.hft.stuttgart.citydoctor2.webservice.model.ValidationStatus;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.FeatureValidationStatus;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.Validation;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.WebBuilding;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.WebCityDoctorModel;

public class ValidationQueries {
	
	private ValidationQueries() {
		// only static use
	}
	
	public static void addFeatureValidationStatus(WebBuilding b, Validation v, FeatureValidationStatus s, EntityManager manager) {
		try {
			manager.getTransaction().begin();
			b.getErrorStatus().add(s);
			v.getErrorStatus().add(s);
			manager.persist(s);
		} finally {
			manager.getTransaction().commit();
		}
	}
	
	public static void persistValidation(EntityManager manager, WebCityDoctorModel model, Validation validation) {
		try {
			manager.getTransaction().begin();
			manager.persist(validation);
			model.getValidations().add(validation);
		} finally {
			manager.getTransaction().commit();
		}
	}

	public static void updateStatus(Validation valid, ValidationStatus status, EntityManager manager) {
		try {
			manager.getTransaction().begin();
			valid.setStatus(status);
		} finally {
			manager.getTransaction().commit();
		}
	}

	public static void updateValidationWithReports(Validation validation, byte[] pdfArray,
			byte[] xmlArray, EntityManager manager) {
		try {
			manager.getTransaction().begin();
			validation.setPdfReport(pdfArray);
			validation.setXmlReport(xmlArray);
		} finally {
			manager.getTransaction().commit();
		}
		
	}

	public static Validation getValidation(int validationId, EntityManager manager) {
		return manager.find(Validation.class, validationId);
	}

	public static boolean hasFeatureErrors(WebBuilding b, Validation validation, EntityManager manager) {
		try {
			manager.getTransaction().begin();
			
			Query query = manager.createQuery("SELECT 1 FROM WebError WHERE EXISTS (SELECT 1 FROM WebError WHERE building = ?1 AND validation = ?2)");
			query.setParameter(1, b);
			query.setParameter(2, validation);
			return !query.getResultList().isEmpty();
		} finally {
			manager.getTransaction().commit();
		}
	}

}
